$LOAD_PATH.unshift __dir__
require 'rutex/version'
require 'tempfile'
require 'mkmf'
require 'mini_magick'

module MakeMakefile
  module Logging
    @logfile = File::NULL
  end
end

class String
  def remove_extension
    sub(/\.[^.]*$/,'')
  end
end

# Generate images from TeX equations
module Rutex
  class RutexError < StandardError; end # :nodoc:
  class PdfError < RutexError; end # :nodoc:
  class PackageError < RutexError; end # :nodoc:

  extend self

  # input equation
  attr_accessor :equation

  # color of equation
  attr_accessor :color

  attr_accessor :output_file # :nodoc:

  PNG_TRANSPARENT=true
  PNG_DENSITY=600
  PNG_QUALITY=100

  def entry(args) # :nodoc:
    unless pdflatex
      raise PdfError, "Missing pdflatex. Install texlive package"
    end

    help if (args.include?('-h') or args.include?('--help'))

    version if args.include?('-v')

    if args.include?('-f')
      _, @output_file = args.slice!(args.index('-f'),2)
    end
    @output_file||=Tempfile.open('equation') {|f| f.path}
    @output_file=@output_file.remove_extension

    if args.include?('-c')
      _, @color = args.slice!(args.index('-c'),2)
    end
    @color||='black'

    help if args.empty?

    @equation=args.join
    make_tex_file
    pdflatex_run
    puts "Generating #{png_file}"
    pdf_to_png
  rescue => e
    abort e.message
  end

  def version # :nodoc:
    puts "rutex #{VERSION}"
    exit
  end

  # returns location of pdflatex binary
  def pdflatex
    Logging.quiet=true
    find_executable 'pdflatex'
  end

  def pdflatex_options # :nodoc:
    "-halt-on-error -interaction=nonstopmode -output-directory=#{tmp_dir}"
  end

  # run pdflatex command with +pdflatex_options+
  def pdflatex_run
    %x(pdflatex #{pdflatex_options} #{tex_file})
    raise PdfError "Errors logged in #{tmp_dir}" unless $?.exitstatus==0
  rescue PdfError => e
    puts "PdfError (#{e.message})"
    abort "Try installing standalone package https://www.ctan.org/pkg/standalone"
  end

  def help # :nodoc:
    puts <<~eof
    Usage:
      rutex <equation> [options]

    Options:
      -c, --color         set color of equation (default black)
      -f, --file          set output file
      -h, --help          print this help message
      -v, --version       print rutex version

    Examples:
      rutex E = mc^2
      rutex '2\\neq\\sqrt{9}'
      rutex 3=\\\\sqrt{9} -c white -f myequation.png
    eof
    exit
  end

  def tmp_dir # :nodoc:
    File.dirname(Tempfile.new)
  end

  # create tex file containing the +equation+
  def make_tex_file
    tex_string = <<~str
    \\documentclass[preview]{standalone}
    \\usepackage{xcolor}
    \\begin{document}
    $\\color{#{color}}#{equation}$
    \\end{document}
    str
    File.write(tex_file, tex_string)
  end

  %w[tex pdf].each do |format|
    define_method("#{format}_file") do
      File.join(tmp_dir,"#{File.basename(output_file)}.#{format}")
    end
  end

  def png_file
    "#{output_file}.png"
  end

  def pdf_to_png
    pdf = MiniMagick::Image.open pdf_file
    MiniMagick::Tool::Convert.new do |convert|
      unless PNG_TRANSPARENT
        convert.background "white"
        convert.flatten
      end
      convert.density PNG_DENSITY
      convert.quality PNG_QUALITY
      convert << pdf.pages.first.path
      convert << "png8:#{png_file}"
    end
  rescue => e
    puts "#{e.message}"
    exit
  end
end
